#include <iostream>
#ifdef EARLY_TEMPLATE
#define __EXECUTABLE__
#ifdef __GNUG__
#include <raslib/template_inst.hh>
#endif
#endif
#include <rasdaman.hh>
#include <rasodmg/database.hh>
#include <rasodmg/partinsert.hh>

#include "../common/src/logging/easylogging++.hh"
_INITIALIZE_EASYLOGGINGPP

int main(int argc, char **argv){
  easyloggingpp::Configurations defaultConf;
  defaultConf.setToDefault();
#ifndef VERBOSE
  defaultConf.setAll(easyloggingpp::ConfigurationType::ToStandardOutput, "false");
  defaultConf.setAll(easyloggingpp::ConfigurationType::Enabled, "false");
#endif
  easyloggingpp::Loggers::reconfigureAllLoggers(defaultConf);

  std::cout << "Insert data: ";

  r_Minterval domain = r_Minterval(3)
     << r_Sinterval((r_Range) 0,(r_Range) 1)
    << r_Sinterval((r_Range) 0, (r_Range) 1)
    << r_Sinterval((r_Range) 0, (r_Range) 1);

  r_GMarray * data = reinterpret_cast<r_GMarray*>(new r_Marray<r_Float>(domain));
  data->set_array_size(2 * 2 * 2 * sizeof(r_Float));
  float *grid = reinterpret_cast<float*>(data->get_array());
  for(std::size_t i = 0;i < 8;i++ ){
    grid[i] = i;
  }
  r_Database db;
  db.set_servername("localhost", 7001);
  db.set_useridentification("rasadmin", "rasadmin");
  db.open("RASBASE");
  r_Partial_Insert insert(db, "test", "FloatCube", "FloatSet3", domain, sizeof(float));
  insert.update(data);
  delete data;
  std::cout << "OK" << std::endl;

  std::cout << "Query data: ";
  r_Transaction t;
  r_OQL_Query q("select t from test as t");
  t.begin(r_Transaction::read_only);
  r_Set<r_Ref_Any > returned;
  r_oql_execute(q, returned);
  auto iter = returned.create_iterator();
  iter.reset();
  if(iter.not_done()){
    if(returned.get_type_schema()->type_id() == r_Type::COLLECTIONTYPE) {
      r_Ref<r_GMarray > grid = (*iter);
      float *values = reinterpret_cast<float*>(grid->get_array());
      bool ok = true;
      for(std::size_t i = 0;i < grid->get_array_size() / sizeof(float);i++ ){
        if(values[i] != i){
          if(i == 0){
            std::cout << std::endl;
          }
          std::cout << "\tvalue at position " << i << "did not match" << std::endl;
          ok = false;
        }
      }
      t.commit();
      if(ok){
        std::cout << "OK" << std::endl;
      }
    }else{
      t.commit();
      std::cout << "did not return a collection" << std::endl;
    }
  }else{
    t.commit();
    std::cout << "returned nothing" << std::endl;
  }
 /*   std::cout << "drop collection: ";
  t.begin();
  r_OQL_Query d("drop collection test");
  r_oql_execute(d);
  t.commit();
  std::cout << "OK" << std::endl;
  db.close();*/


  return 0;
}
